@@ -1,15 +1,18 @@
 """
 Utility for running a prompt_toolkit application in an asyncssh server.
 """
+from __future__ import annotations
+
 import asyncio
 import traceback
-from typing import Any, Awaitable, Callable, Optional, TextIO, cast
+from asyncio import get_running_loop
+from typing import Any, Awaitable, Callable, TextIO, cast
 
 import asyncssh
 
 from prompt_toolkit.application.current import AppSession, create_app_session
 from prompt_toolkit.data_structures import Size
-from prompt_toolkit.input import create_pipe_input
+from prompt_toolkit.input import PipeInput, create_pipe_input
 from prompt_toolkit.output.vt100 import Vt100_Output
 
 __all__ = ["PromptToolkitSSHSession", "PromptToolkitSSHServer"]
@@ -17,18 +20,22 @@ __all__ = ["PromptToolkitSSHSession", "PromptToolkitSSHServer"]
 
 class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
     def __init__(
-        self, interact: Callable[["PromptToolkitSSHSession"], Awaitable[None]]
+        self,
+        interact: Callable[[PromptToolkitSSHSession], Awaitable[None]],
+        *,
+        enable_cpr: bool,
     ) -> None:
         self.interact = interact
-        self.interact_task: Optional[asyncio.Task[None]] = None
-        self._chan: Optional[Any] = None
-        self.app_session: Optional[AppSession] = None
+        self.enable_cpr = enable_cpr
+        self.interact_task: asyncio.Task[None] | None = None
+        self._chan: Any | None = None
+        self.app_session: AppSession | None = None
 
         # PipInput object, for sending input in the CLI.
         # (This is something that we can use in the prompt_toolkit event loop,
         # but still write date in manually.)
-        self._input = create_pipe_input()
-        self._output: Optional[Vt100_Output] = None
+        self._input: PipeInput | None = None
+        self._output: Vt100_Output | None = None
 
         # Output object. Don't render to the real stdout, but write everything
         # in the SSH channel.
@@ -70,7 +77,7 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
         return True
 
     def session_started(self) -> None:
-        self.interact_task = asyncio.get_event_loop().create_task(self._interact())
+        self.interact_task = get_running_loop().create_task(self._interact())
 
     async def _interact(self) -> None:
         if self._chan is None:
@@ -85,18 +92,20 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
         term = self._chan.get_terminal_type()
 
         self._output = Vt100_Output(
-            self.stdout, self._get_size, term=term, write_binary=False
+            self.stdout, self._get_size, term=term, enable_cpr=self.enable_cpr
         )
-        with create_app_session(input=self._input, output=self._output) as session:
-            self.app_session = session
-            try:
-                await self.interact(self)
-            except BaseException:
-                traceback.print_exc()
-            finally:
-                # Close the connection.
-                self._chan.close()
-                self._input.close()
+
+        with create_pipe_input() as self._input:
+            with create_app_session(input=self._input, output=self._output) as session:
+                self.app_session = session
+                try:
+                    await self.interact(self)
+                except BaseException:
+                    traceback.print_exc()
+                finally:
+                    # Close the connection.
+                    self._chan.close()
+                    self._input.close()
 
     def terminal_size_changed(
         self, width: int, height: int, pixwidth: object, pixheight: object
@@ -106,10 +115,14 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
             self.app_session.app._on_resize()
 
     def data_received(self, data: str, datatype: object) -> None:
+        if self._input is None:
+            # Should not happen.
+            return
+
         self._input.send_text(data)
 
 
-class PromptToolkitSSHServer(asyncssh.SSHServer):  # type: ignore
+class PromptToolkitSSHServer(asyncssh.SSHServer):
     """
     Run a prompt_toolkit application over an asyncssh server.
 
@@ -130,7 +143,7 @@ class PromptToolkitSSHServer(asyncssh.SSHServer):  # type: ignore
             print_formatted_text('You said: ', text)
 
         server = PromptToolkitSSHServer(interact=interact)
-        loop = get_event_loop()
+        loop = get_running_loop()
         loop.run_until_complete(
             asyncssh.create_server(
                 lambda: MySSHServer(interact),
@@ -140,16 +153,25 @@ class PromptToolkitSSHServer(asyncssh.SSHServer):  # type: ignore
             )
         )
         loop.run_forever()
+
+    :param enable_cpr: When `True`, the default, try to detect whether the SSH
+        client runs in a terminal that responds to "cursor position requests".
+        That way, we can properly determine how much space there is available
+        for the UI (especially for drop down menus) to render.
     """
 
     def __init__(
-        self, interact: Callable[[PromptToolkitSSHSession], Awaitable[None]]
+        self,
+        interact: Callable[[PromptToolkitSSHSession], Awaitable[None]],
+        *,
+        enable_cpr: bool = True,
     ) -> None:
         self.interact = interact
+        self.enable_cpr = enable_cpr
 
     def begin_auth(self, username: str) -> bool:
         # No authentication.
         return False
 
     def session_requested(self) -> PromptToolkitSSHSession:
-        return PromptToolkitSSHSession(self.interact)
+        return PromptToolkitSSHSession(self.interact, enable_cpr=self.enable_cpr)
