@@ -2,20 +2,12 @@
 Container for the layout.
 (Containers can contain other containers or user interface controls.)
 """
+from __future__ import annotations
+
 from abc import ABCMeta, abstractmethod
 from enum import Enum
 from functools import partial
-from typing import (
-    TYPE_CHECKING,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
-    cast,
-)
+from typing import TYPE_CHECKING, Callable, Sequence, Union, cast
 
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit.cache import SimpleCache
@@ -59,9 +51,10 @@ from .screen import _CHAR_CACHE, Screen, WritePosition
 from .utils import explode_text_fragments
 
 if TYPE_CHECKING:
-    from typing_extensions import Protocol
+    from typing_extensions import Protocol, TypeGuard
+
+    from prompt_toolkit.key_binding.key_bindings import NotImplementedOrNone
 
-    NotImplementedOrNone = object
 
 __all__ = [
     "AnyContainer",
@@ -119,7 +112,7 @@ class Container(metaclass=ABCMeta):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         """
         Write the actual content to the screen.
@@ -140,7 +133,7 @@ class Container(metaclass=ABCMeta):
         """
         return False
 
-    def get_key_bindings(self) -> Optional[KeyBindingsBase]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         """
         Returns a :class:`.KeyBindings` object. These bindings become active when any
         user control in this container has the focus, except if any containers
@@ -149,7 +142,7 @@ class Container(metaclass=ABCMeta):
         return None
 
     @abstractmethod
-    def get_children(self) -> List["Container"]:
+    def get_children(self) -> list[Container]:
         """
         Return the list of child :class:`.Container` objects.
         """
@@ -163,22 +156,23 @@ if TYPE_CHECKING:
         Any object that implements ``__pt_container__`` represents a container.
         """
 
-        def __pt_container__(self) -> "AnyContainer":
+        def __pt_container__(self) -> AnyContainer:
             ...
 
 
 AnyContainer = Union[Container, "MagicContainer"]
 
 
-def _window_too_small() -> "Window":
-    " Create a `Window` that displays the 'Window too small' text. "
+def _window_too_small() -> Window:
+    "Create a `Window` that displays the 'Window too small' text."
     return Window(
         FormattedTextControl(text=[("class:window-too-small", " Window too small... ")])
     )
 
 
 class VerticalAlign(Enum):
-    " Alignment for `HSplit`. "
+    "Alignment for `HSplit`."
+
     TOP = "TOP"
     CENTER = "CENTER"
     BOTTOM = "BOTTOM"
@@ -186,7 +180,8 @@ class VerticalAlign(Enum):
 
 
 class HorizontalAlign(Enum):
-    " Alignment for `VSplit`. "
+    "Alignment for `VSplit`."
+
     LEFT = "LEFT"
     CENTER = "CENTER"
     RIGHT = "RIGHT"
@@ -201,18 +196,17 @@ class _Split(Container):
     def __init__(
         self,
         children: Sequence[AnyContainer],
-        window_too_small: Optional[Container] = None,
+        window_too_small: Container | None = None,
         padding: AnyDimension = Dimension.exact(0),
-        padding_char: Optional[str] = None,
+        padding_char: str | None = None,
         padding_style: str = "",
         width: AnyDimension = None,
         height: AnyDimension = None,
-        z_index: Optional[int] = None,
+        z_index: int | None = None,
         modal: bool = False,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        style: Union[str, Callable[[], str]] = "",
+        key_bindings: KeyBindingsBase | None = None,
+        style: str | Callable[[], str] = "",
     ) -> None:
-
         self.children = [to_container(c) for c in children]
         self.window_too_small = window_too_small or _window_too_small()
         self.padding = padding
@@ -230,10 +224,10 @@ class _Split(Container):
     def is_modal(self) -> bool:
         return self.modal
 
-    def get_key_bindings(self) -> Optional[KeyBindingsBase]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         return self.key_bindings
 
-    def get_children(self) -> List[Container]:
+    def get_children(self) -> list[Container]:
         return self.children
 
 
@@ -274,19 +268,18 @@ class HSplit(_Split):
     def __init__(
         self,
         children: Sequence[AnyContainer],
-        window_too_small: Optional[Container] = None,
+        window_too_small: Container | None = None,
         align: VerticalAlign = VerticalAlign.JUSTIFY,
         padding: AnyDimension = 0,
-        padding_char: Optional[str] = None,
+        padding_char: str | None = None,
         padding_style: str = "",
         width: AnyDimension = None,
         height: AnyDimension = None,
-        z_index: Optional[int] = None,
+        z_index: int | None = None,
         modal: bool = False,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        style: Union[str, Callable[[], str]] = "",
+        key_bindings: KeyBindingsBase | None = None,
+        style: str | Callable[[], str] = "",
     ) -> None:
-
         super().__init__(
             children=children,
             window_too_small=window_too_small,
@@ -304,7 +297,7 @@ class HSplit(_Split):
         self.align = align
 
         self._children_cache: SimpleCache[
-            Tuple[Container, ...], List[Container]
+            tuple[Container, ...], list[Container]
         ] = SimpleCache(maxsize=1)
         self._remaining_space_window = Window()  # Dummy window.
 
@@ -332,13 +325,13 @@ class HSplit(_Split):
             c.reset()
 
     @property
-    def _all_children(self) -> List[Container]:
+    def _all_children(self) -> list[Container]:
         """
         List of child objects, including padding.
         """
 
-        def get() -> List[Container]:
-            result: List[Container] = []
+        def get() -> list[Container]:
+            result: list[Container] = []
 
             # Padding Top.
             if self.align in (VerticalAlign.CENTER, VerticalAlign.BOTTOM):
@@ -372,7 +365,7 @@ class HSplit(_Split):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         """
         Render the prompt to a `Screen` instance.
@@ -422,7 +415,7 @@ class HSplit(_Split):
                     z_index,
                 )
 
-    def _divide_heights(self, write_position: WritePosition) -> Optional[List[int]]:
+    def _divide_heights(self, write_position: WritePosition) -> list[int] | None:
         """
         Return the heights for all rows.
         Or None when there is not enough space.
@@ -512,19 +505,18 @@ class VSplit(_Split):
     def __init__(
         self,
         children: Sequence[AnyContainer],
-        window_too_small: Optional[Container] = None,
+        window_too_small: Container | None = None,
         align: HorizontalAlign = HorizontalAlign.JUSTIFY,
         padding: AnyDimension = 0,
-        padding_char: Optional[str] = None,
+        padding_char: str | None = None,
         padding_style: str = "",
         width: AnyDimension = None,
         height: AnyDimension = None,
-        z_index: Optional[int] = None,
+        z_index: int | None = None,
         modal: bool = False,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        style: Union[str, Callable[[], str]] = "",
+        key_bindings: KeyBindingsBase | None = None,
+        style: str | Callable[[], str] = "",
     ) -> None:
-
         super().__init__(
             children=children,
             window_too_small=window_too_small,
@@ -542,7 +534,7 @@ class VSplit(_Split):
         self.align = align
 
         self._children_cache: SimpleCache[
-            Tuple[Container, ...], List[Container]
+            tuple[Container, ...], list[Container]
         ] = SimpleCache(maxsize=1)
         self._remaining_space_window = Window()  # Dummy window.
 
@@ -585,13 +577,13 @@ class VSplit(_Split):
             c.reset()
 
     @property
-    def _all_children(self) -> List[Container]:
+    def _all_children(self) -> list[Container]:
         """
         List of child objects, including padding.
         """
 
-        def get() -> List[Container]:
-            result: List[Container] = []
+        def get() -> list[Container]:
+            result: list[Container] = []
 
             # Padding left.
             if self.align in (HorizontalAlign.CENTER, HorizontalAlign.RIGHT):
@@ -618,7 +610,7 @@ class VSplit(_Split):
 
         return self._children_cache.get(tuple(self.children), get)
 
-    def _divide_widths(self, width: int) -> Optional[List[int]]:
+    def _divide_widths(self, width: int) -> list[int] | None:
         """
         Return the widths for all columns.
         Or None when there is not enough space.
@@ -676,7 +668,7 @@ class VSplit(_Split):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         """
         Render the prompt to a `Screen` instance.
@@ -751,7 +743,7 @@ class FloatContainer(Container):
                        floats=[
                            Float(xcursor=True,
                                 ycursor=True,
-                                content=CompletionMenu(...))
+                                content=CompletionsMenu(...))
                        ])
 
     :param z_index: (int or None) When specified, this can be used to bring
@@ -762,13 +754,12 @@ class FloatContainer(Container):
     def __init__(
         self,
         content: AnyContainer,
-        floats: List["Float"],
+        floats: list[Float],
         modal: bool = False,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        style: Union[str, Callable[[], str]] = "",
-        z_index: Optional[int] = None,
+        key_bindings: KeyBindingsBase | None = None,
+        style: str | Callable[[], str] = "",
+        z_index: int | None = None,
     ) -> None:
-
         self.content = to_container(content)
         self.floats = floats
 
@@ -801,7 +792,7 @@ class FloatContainer(Container):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         style = parent_style + " " + to_str(self.style)
         z_index = z_index if self.z_index is None else self.z_index
@@ -825,7 +816,7 @@ class FloatContainer(Container):
 
             if postpone:
                 new_z_index = (
-                    number + 10 ** 8
+                    number + 10**8
                 )  # Draw as late as possible, but keep the order.
                 screen.draw_with_z_index(
                     z_index=new_z_index,
@@ -853,15 +844,15 @@ class FloatContainer(Container):
 
     def _draw_float(
         self,
-        fl: "Float",
+        fl: Float,
         screen: Screen,
         mouse_handlers: MouseHandlers,
         write_position: WritePosition,
         style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
-        " Draw a single Float. "
+        "Draw a single Float."
         # When a menu_position was given, use this instead of the cursor
         # position. (These cursor positions are absolute, translate again
         # relative to the write_position.)
@@ -1016,10 +1007,10 @@ class FloatContainer(Container):
     def is_modal(self) -> bool:
         return self.modal
 
-    def get_key_bindings(self) -> Optional[KeyBindingsBase]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         return self.key_bindings
 
-    def get_children(self) -> List[Container]:
+    def get_children(self) -> list[Container]:
         children = [self.content]
         children.extend(f.content for f in self.floats)
         return children
@@ -1054,21 +1045,20 @@ class Float:
     def __init__(
         self,
         content: AnyContainer,
-        top: Optional[int] = None,
-        right: Optional[int] = None,
-        bottom: Optional[int] = None,
-        left: Optional[int] = None,
-        width: Optional[Union[int, Callable[[], int]]] = None,
-        height: Optional[Union[int, Callable[[], int]]] = None,
+        top: int | None = None,
+        right: int | None = None,
+        bottom: int | None = None,
+        left: int | None = None,
+        width: int | Callable[[], int] | None = None,
+        height: int | Callable[[], int] | None = None,
         xcursor: bool = False,
         ycursor: bool = False,
-        attach_to_window: Optional[AnyContainer] = None,
+        attach_to_window: AnyContainer | None = None,
         hide_when_covering_content: bool = False,
         allow_cover_cursor: bool = False,
         z_index: int = 1,
         transparent: bool = False,
     ) -> None:
-
         assert z_index >= 1
 
         self.left = left
@@ -1092,12 +1082,12 @@ class Float:
         self.z_index = z_index
         self.transparent = to_filter(transparent)
 
-    def get_width(self) -> Optional[int]:
+    def get_width(self) -> int | None:
         if callable(self.width):
             return self.width()
         return self.width
 
-    def get_height(self) -> Optional[int]:
+    def get_height(self) -> int | None:
         if callable(self.height):
             return self.height()
         return self.height
@@ -1135,20 +1125,19 @@ class WindowRenderInfo:
 
     def __init__(
         self,
-        window: "Window",
+        window: Window,
         ui_content: UIContent,
         horizontal_scroll: int,
         vertical_scroll: int,
         window_width: int,
         window_height: int,
-        configured_scroll_offsets: "ScrollOffsets",
-        visible_line_to_row_col: Dict[int, Tuple[int, int]],
-        rowcol_to_yx: Dict[Tuple[int, int], Tuple[int, int]],
+        configured_scroll_offsets: ScrollOffsets,
+        visible_line_to_row_col: dict[int, tuple[int, int]],
+        rowcol_to_yx: dict[tuple[int, int], tuple[int, int]],
         x_offset: int,
         y_offset: int,
         wrap_lines: bool,
     ) -> None:
-
         self.window = window
         self.ui_content = ui_content
         self.vertical_scroll = vertical_scroll
@@ -1165,7 +1154,7 @@ class WindowRenderInfo:
         self._y_offset = y_offset
 
     @property
-    def visible_line_to_input_line(self) -> Dict[int, int]:
+    def visible_line_to_input_line(self) -> dict[int, int]:
         return {
             visible_line: rowcol[0]
             for visible_line, rowcol in self.visible_line_to_row_col.items()
@@ -1188,7 +1177,7 @@ class WindowRenderInfo:
             return Point(x=x - self._x_offset, y=y - self._y_offset)
 
     @property
-    def applied_scroll_offsets(self) -> "ScrollOffsets":
+    def applied_scroll_offsets(self) -> ScrollOffsets:
         """
         Return a :class:`.ScrollOffsets` instance that indicates the actual
         offset. This can be less than or equal to what's configured. E.g, when
@@ -1216,7 +1205,7 @@ class WindowRenderInfo:
         )
 
     @property
-    def displayed_lines(self) -> List[int]:
+    def displayed_lines(self) -> list[int]:
         """
         List of all the visible rows. (Line numbers of the input buffer.)
         The last line may not be entirely visible.
@@ -1224,13 +1213,13 @@ class WindowRenderInfo:
         return sorted(row for row, col in self.visible_line_to_row_col.values())
 
     @property
-    def input_line_to_visible_line(self) -> Dict[int, int]:
+    def input_line_to_visible_line(self) -> dict[int, int]:
         """
         Return the dictionary mapping the line numbers of the input buffer to
         the lines of the screen. When a line spans several rows at the screen,
         the first row appears in the dictionary.
         """
-        result: Dict[int, int] = {}
+        result: dict[int, int] = {}
         for k, v in self.visible_line_to_input_line.items():
             if v in result:
                 result[v] = min(result[v], k)
@@ -1336,12 +1325,11 @@ class ScrollOffsets:
 
     def __init__(
         self,
-        top: Union[int, Callable[[], int]] = 0,
-        bottom: Union[int, Callable[[], int]] = 0,
-        left: Union[int, Callable[[], int]] = 0,
-        right: Union[int, Callable[[], int]] = 0,
+        top: int | Callable[[], int] = 0,
+        bottom: int | Callable[[], int] = 0,
+        left: int | Callable[[], int] = 0,
+        right: int | Callable[[], int] = 0,
     ) -> None:
-
         self._top = top
         self._bottom = bottom
         self._left = left
@@ -1364,7 +1352,7 @@ class ScrollOffsets:
         return to_int(self._right)
 
     def __repr__(self) -> str:
-        return "ScrollOffsets(top=%r, bottom=%r, left=%r, right=%r)" % (
+        return "ScrollOffsets(top={!r}, bottom={!r}, left={!r}, right={!r})".format(
             self._top,
             self._bottom,
             self._left,
@@ -1463,33 +1451,32 @@ class Window(Container):
 
     def __init__(
         self,
-        content: Optional[UIControl] = None,
+        content: UIControl | None = None,
         width: AnyDimension = None,
         height: AnyDimension = None,
-        z_index: Optional[int] = None,
+        z_index: int | None = None,
         dont_extend_width: FilterOrBool = False,
         dont_extend_height: FilterOrBool = False,
         ignore_content_width: FilterOrBool = False,
         ignore_content_height: FilterOrBool = False,
-        left_margins: Optional[Sequence[Margin]] = None,
-        right_margins: Optional[Sequence[Margin]] = None,
-        scroll_offsets: Optional[ScrollOffsets] = None,
+        left_margins: Sequence[Margin] | None = None,
+        right_margins: Sequence[Margin] | None = None,
+        scroll_offsets: ScrollOffsets | None = None,
         allow_scroll_beyond_bottom: FilterOrBool = False,
         wrap_lines: FilterOrBool = False,
-        get_vertical_scroll: Optional[Callable[["Window"], int]] = None,
-        get_horizontal_scroll: Optional[Callable[["Window"], int]] = None,
+        get_vertical_scroll: Callable[[Window], int] | None = None,
+        get_horizontal_scroll: Callable[[Window], int] | None = None,
         always_hide_cursor: FilterOrBool = False,
         cursorline: FilterOrBool = False,
         cursorcolumn: FilterOrBool = False,
-        colorcolumns: Union[
-            None, List[ColorColumn], Callable[[], List[ColorColumn]]
-        ] = None,
-        align: Union[WindowAlign, Callable[[], WindowAlign]] = WindowAlign.LEFT,
-        style: Union[str, Callable[[], str]] = "",
-        char: Union[None, str, Callable[[], str]] = None,
-        get_line_prefix: Optional[GetLinePrefixCallable] = None,
+        colorcolumns: (
+            None | list[ColorColumn] | Callable[[], list[ColorColumn]]
+        ) = None,
+        align: WindowAlign | Callable[[], WindowAlign] = WindowAlign.LEFT,
+        style: str | Callable[[], str] = "",
+        char: None | str | Callable[[], str] = None,
+        get_line_prefix: GetLinePrefixCallable | None = None,
     ) -> None:
-
         self.allow_scroll_beyond_bottom = to_filter(allow_scroll_beyond_bottom)
         self.always_hide_cursor = to_filter(always_hide_cursor)
         self.wrap_lines = to_filter(wrap_lines)
@@ -1518,9 +1505,9 @@ class Window(Container):
 
         # Cache for the screens generated by the margin.
         self._ui_content_cache: SimpleCache[
-            Tuple[int, int, int], UIContent
+            tuple[int, int, int], UIContent
         ] = SimpleCache(maxsize=8)
-        self._margin_width_cache: SimpleCache[Tuple[Margin, int], int] = SimpleCache(
+        self._margin_width_cache: SimpleCache[tuple[Margin, int], int] = SimpleCache(
             maxsize=1
         )
 
@@ -1543,13 +1530,14 @@ class Window(Container):
 
         #: Keep render information (mappings between buffer input and render
         #: output.)
-        self.render_info: Optional[WindowRenderInfo] = None
+        self.render_info: WindowRenderInfo | None = None
 
     def _get_margin_width(self, margin: Margin) -> int:
         """
         Return the width for this margin.
         (Calculate only once per render time.)
         """
+
         # Margin.get_width, needs to have a UIContent instance.
         def get_ui_content() -> UIContent:
             return self._get_ui_content(width=0, height=0)
@@ -1573,7 +1561,7 @@ class Window(Container):
         Calculate the preferred width for this window.
         """
 
-        def preferred_content_width() -> Optional[int]:
+        def preferred_content_width() -> int | None:
             """Content width: is only calculated if no exact width for the
             window was given."""
             if self.ignore_content_width():
@@ -1604,7 +1592,7 @@ class Window(Container):
         Calculate the preferred height for this window.
         """
 
-        def preferred_content_height() -> Optional[int]:
+        def preferred_content_height() -> int | None:
             """Content height: is only calculated if no exact height for the
             window was given."""
             if self.ignore_content_height():
@@ -1628,8 +1616,8 @@ class Window(Container):
 
     @staticmethod
     def _merge_dimensions(
-        dimension: Optional[Dimension],
-        get_preferred: Callable[[], Optional[int]],
+        dimension: Dimension | None,
+        get_preferred: Callable[[], int | None],
         dont_extend: bool = False,
     ) -> Dimension:
         """
@@ -1641,7 +1629,7 @@ class Window(Container):
 
         # When a preferred dimension was explicitly given to the Window,
         # ignore the UIControl.
-        preferred: Optional[int]
+        preferred: int | None
 
         if dimension.preferred_specified:
             preferred = dimension.preferred
@@ -1661,8 +1649,8 @@ class Window(Container):
 
         # When a `dont_extend` flag has been given, use the preferred dimension
         # also as the max dimension.
-        max_: Optional[int]
-        min_: Optional[int]
+        max_: int | None
+        min_: int | None
 
         if dont_extend and preferred is not None:
             max_ = min(dimension.max, preferred)
@@ -1686,8 +1674,8 @@ class Window(Container):
         key = (get_app().render_counter, width, height)
         return self._ui_content_cache.get(key, get_content)
 
-    def _get_digraph_char(self) -> Optional[str]:
-        " Return `False`, or the Digraph symbol to be used. "
+    def _get_digraph_char(self) -> str | None:
+        "Return `False`, or the Digraph symbol to be used."
         app = get_app()
         if app.quoted_insert:
             return "^"
@@ -1704,7 +1692,7 @@ class Window(Container):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         """
         Write window to screen. This renders the user control, the margins and
@@ -1829,13 +1817,16 @@ class Window(Container):
         self.render_info = render_info
 
         # Set mouse handlers.
-        def mouse_handler(mouse_event: MouseEvent) -> None:
-            """Wrapper around the mouse_handler of the `UIControl` that turns
-            screen coordinates into line coordinates."""
+        def mouse_handler(mouse_event: MouseEvent) -> NotImplementedOrNone:
+            """
+            Wrapper around the mouse_handler of the `UIControl` that turns
+            screen coordinates into line coordinates.
+            Returns `NotImplemented` if no UI invalidation should be done.
+            """
             # Don't handle mouse events outside of the current modal part of
             # the UI.
             if self not in get_app().layout.walk_through_modal_area():
-                return
+                return NotImplemented
 
             # Find row/col position first.
             yx_to_rowcol = {v: k for k, v in rowcol_to_yx.items()}
@@ -1861,6 +1852,8 @@ class Window(Container):
                         MouseEvent(
                             position=Point(x=col, y=row),
                             event_type=mouse_event.event_type,
+                            button=mouse_event.button,
+                            modifiers=mouse_event.modifiers,
                         )
                     )
                     break
@@ -1871,13 +1864,18 @@ class Window(Container):
                 # Report (0,0) instead.)
                 result = self.content.mouse_handler(
                     MouseEvent(
-                        position=Point(x=0, y=0), event_type=mouse_event.event_type
+                        position=Point(x=0, y=0),
+                        event_type=mouse_event.event_type,
+                        button=mouse_event.button,
+                        modifiers=mouse_event.modifiers,
                     )
                 )
 
             # If it returns NotImplemented, handle it here.
             if result == NotImplemented:
-                self._mouse_handler(mouse_event)
+                result = self._mouse_handler(mouse_event)
+
+            return result
 
         mouse_handlers.set_mouse_handler_for_range(
             x_min=write_position.xpos + sum(left_margin_widths),
@@ -1891,7 +1889,7 @@ class Window(Container):
         move_x = 0
 
         def render_margin(m: Margin, width: int) -> UIContent:
-            " Render margin. Return `Screen`. "
+            "Render margin. Return `Screen`."
             # Retrieve margin fragments.
             fragments = m.create_margin(render_info, width, write_position.height)
 
@@ -1942,8 +1940,8 @@ class Window(Container):
         always_hide_cursor: bool = False,
         has_focus: bool = False,
         align: WindowAlign = WindowAlign.LEFT,
-        get_line_prefix: Optional[Callable[[int, int], AnyFormattedText]] = None,
-    ) -> Tuple[Dict[int, Tuple[int, int]], Dict[Tuple[int, int], Tuple[int, int]]]:
+        get_line_prefix: Callable[[int, int], AnyFormattedText] | None = None,
+    ) -> tuple[dict[int, tuple[int, int]], dict[tuple[int, int], tuple[int, int]]]:
         """
         Copy the UIContent into the output screen.
         Return (visible_line_to_row_col, rowcol_to_yx) tuple.
@@ -1959,10 +1957,10 @@ class Window(Container):
 
         # Map visible line number to (row, col) of input.
         # 'col' will always be zero if line wrapping is off.
-        visible_line_to_row_col: Dict[int, Tuple[int, int]] = {}
+        visible_line_to_row_col: dict[int, tuple[int, int]] = {}
 
         # Maps (row, col) from the input to (y, x) screen coordinates.
-        rowcol_to_yx: Dict[Tuple[int, int], Tuple[int, int]] = {}
+        rowcol_to_yx: dict[tuple[int, int], tuple[int, int]] = {}
 
         def copy_line(
             line: StyleAndTextTuples,
@@ -1970,7 +1968,7 @@ class Window(Container):
             x: int,
             y: int,
             is_input: bool = False,
-        ) -> Tuple[int, int]:
+        ) -> tuple[int, int]:
             """
             Copy over a single line to the output screen. This can wrap over
             multiple lines in the output. It will call the prefix (prompt)
@@ -2048,7 +2046,7 @@ class Window(Container):
                             return x, y  # Break out of all for loops.
 
                     # Set character in screen and shift 'x'.
-                    if x >= 0 and y >= 0 and x < write_position.width:
+                    if x >= 0 and y >= 0 and x < width:
                         new_buffer_row[x + xpos] = char
 
                         # When we print a multi width character, make sure
@@ -2110,7 +2108,7 @@ class Window(Container):
         copy()
 
         def cursor_pos_to_screen_pos(row: int, col: int) -> Point:
-            " Translate row/col from UIContent to real Screen coordinates. "
+            "Translate row/col from UIContent to real Screen coordinates."
             try:
                 y, x = rowcol_to_yx[row, col]
             except KeyError:
@@ -2176,7 +2174,7 @@ class Window(Container):
         Erase/fill the background.
         (Useful for floats and when a `char` has been given.)
         """
-        char: Optional[str]
+        char: str | None
         if callable(self.char):
             char = self.char()
         else:
@@ -2194,7 +2192,6 @@ class Window(Container):
     def _apply_style(
         self, new_screen: Screen, write_position: WritePosition, parent_style: str
     ) -> None:
-
         # Apply `self.style`.
         style = parent_style + " " + to_str(self.style)
 
@@ -2370,7 +2367,7 @@ class Window(Container):
             self.vertical_scroll_2 = min(
                 text_before_height - 1,  # Keep the cursor visible.
                 line_height
-                - height,  # Avoid blank lines at the bottom when scolling up again.
+                - height,  # Avoid blank lines at the bottom when scrolling up again.
                 self.vertical_scroll_2,
             )
             self.vertical_scroll_2 = max(
@@ -2474,7 +2471,7 @@ class Window(Container):
             window_size: int,
             content_size: int,
         ) -> int:
-            " Scrolling algorithm. Used for both horizontal and vertical scrolling. "
+            "Scrolling algorithm. Used for both horizontal and vertical scrolling."
             # Calculate the scroll offset to apply.
             # This can obviously never be more than have the screen size. Also, when the
             # cursor appears at the top or bottom, we don't apply the offset.
@@ -2547,18 +2544,25 @@ class Window(Container):
             ),
         )
 
-    def _mouse_handler(self, mouse_event: MouseEvent) -> None:
+    def _mouse_handler(self, mouse_event: MouseEvent) -> NotImplementedOrNone:
         """
         Mouse handler. Called when the UI control doesn't handle this
         particular event.
+
+        Return `NotImplemented` if nothing was done as a consequence of this
+        key binding (no UI invalidate required in that case).
         """
         if mouse_event.event_type == MouseEventType.SCROLL_DOWN:
             self._scroll_down()
+            return None
         elif mouse_event.event_type == MouseEventType.SCROLL_UP:
             self._scroll_up()
+            return None
+
+        return NotImplemented
 
     def _scroll_down(self) -> None:
-        " Scroll window down. "
+        "Scroll window down."
         info = self.render_info
 
         if info is None:
@@ -2571,7 +2575,7 @@ class Window(Container):
             self.vertical_scroll += 1
 
     def _scroll_up(self) -> None:
-        " Scroll window up. "
+        "Scroll window up."
         info = self.render_info
 
         if info is None:
@@ -2587,10 +2591,10 @@ class Window(Container):
 
             self.vertical_scroll -= 1
 
-    def get_key_bindings(self) -> Optional[KeyBindingsBase]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         return self.content.get_key_bindings()
 
-    def get_children(self) -> List[Container]:
+    def get_children(self) -> list[Container]:
         return []
 
 
@@ -2609,7 +2613,7 @@ class ConditionalContainer(Container):
         self.filter = to_filter(filter)
 
     def __repr__(self) -> str:
-        return "ConditionalContainer(%r, filter=%r)" % (self.content, self.filter)
+        return f"ConditionalContainer({self.content!r}, filter={self.filter!r})"
 
     def reset(self) -> None:
         self.content.reset()
@@ -2633,14 +2637,14 @@ class ConditionalContainer(Container):
         write_position: WritePosition,
         parent_style: str,
         erase_bg: bool,
-        z_index: Optional[int],
+        z_index: int | None,
     ) -> None:
         if self.filter():
             return self.content.write_to_screen(
                 screen, mouse_handlers, write_position, parent_style, erase_bg, z_index
             )
 
-    def get_children(self) -> List[Container]:
+    def get_children(self) -> list[Container]:
         return [self.content]
 
 
@@ -2674,18 +2678,28 @@ class DynamicContainer(Container):
     def preferred_height(self, width: int, max_available_height: int) -> Dimension:
         return self._get_container().preferred_height(width, max_available_height)
 
-    def write_to_screen(self, *a, **kw):
-        self._get_container().write_to_screen(*a, **kw)
+    def write_to_screen(
+        self,
+        screen: Screen,
+        mouse_handlers: MouseHandlers,
+        write_position: WritePosition,
+        parent_style: str,
+        erase_bg: bool,
+        z_index: int | None,
+    ) -> None:
+        self._get_container().write_to_screen(
+            screen, mouse_handlers, write_position, parent_style, erase_bg, z_index
+        )
 
     def is_modal(self) -> bool:
         return False
 
-    def get_key_bindings(self) -> Optional[KeyBindingsBase]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         # Key bindings will be collected when `layout.walk()` finds the child
         # container.
         return None
 
-    def get_children(self) -> List[Container]:
+    def get_children(self) -> list[Container]:
         # Here we have to return the current active container itself, not its
         # children. Otherwise, we run into issues where `layout.walk()` will
         # never see an object of type `Window` if this contains a window. We
@@ -2702,7 +2716,7 @@ def to_container(container: AnyContainer) -> Container:
     elif hasattr(container, "__pt_container__"):
         return to_container(container.__pt_container__())
     else:
-        raise ValueError("Not a container object: %r" % (container,))
+        raise ValueError(f"Not a container object: {container!r}")
 
 
 def to_window(container: AnyContainer) -> Window:
@@ -2714,10 +2728,10 @@ def to_window(container: AnyContainer) -> Window:
     elif hasattr(container, "__pt_container__"):
         return to_window(cast("MagicContainer", container).__pt_container__())
     else:
-        raise ValueError("Not a Window object: %r." % (container,))
+        raise ValueError(f"Not a Window object: {container!r}.")
 
 
-def is_container(value: object) -> bool:
+def is_container(value: object) -> TypeGuard[AnyContainer]:
     """
     Checks whether the given value is a container object
     (for use in assert statements).
