@@ -1,19 +1,11 @@
 """
 User interface Controls for the layout.
 """
+from __future__ import annotations
+
 import time
 from abc import ABCMeta, abstractmethod
-from typing import (
-    TYPE_CHECKING,
-    Callable,
-    Dict,
-    Hashable,
-    Iterable,
-    List,
-    NamedTuple,
-    Optional,
-    Union,
-)
+from typing import TYPE_CHECKING, Callable, Hashable, Iterable, NamedTuple
 
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit.buffer import Buffer
@@ -32,7 +24,7 @@ from prompt_toolkit.formatted_text.utils import (
     split_lines,
 )
 from prompt_toolkit.lexers import Lexer, SimpleLexer
-from prompt_toolkit.mouse_events import MouseEvent, MouseEventType
+from prompt_toolkit.mouse_events import MouseButton, MouseEvent, MouseEventType
 from prompt_toolkit.search import SearchState
 from prompt_toolkit.selection import SelectionType
 from prompt_toolkit.utils import get_cwidth
@@ -48,23 +40,12 @@ from .processors import (
 )
 
 if TYPE_CHECKING:
-    from prompt_toolkit.key_binding.key_bindings import KeyBindingsBase
+    from prompt_toolkit.key_binding.key_bindings import (
+        KeyBindingsBase,
+        NotImplementedOrNone,
+    )
     from prompt_toolkit.utils import Event
 
-    # The only two return values for a mouse hander are `None` and
-    # `NotImplemented`. For the type checker it's best to annotate this as
-    # `object`. (The consumer never expects a more specific instance: checking
-    # for NotImplemented can be done using `is NotImplemented`.)
-    NotImplementedOrNone = object
-    # Other non-working options are:
-    # * Optional[Literal[NotImplemented]]
-    #      --> Doesn't work, Literal can't take an Any.
-    # * None
-    #      --> Doesn't work. We can't assign the result of a function that
-    #          returns `None` to a variable.
-    # * Any
-    #      --> Works, but too broad.
-
 
 __all__ = [
     "BufferControl",
@@ -87,7 +68,7 @@ class UIControl(metaclass=ABCMeta):
         # Default reset. (Doesn't have to be implemented.)
         pass
 
-    def preferred_width(self, max_available_width: int) -> Optional[int]:
+    def preferred_width(self, max_available_width: int) -> int | None:
         return None
 
     def preferred_height(
@@ -95,8 +76,8 @@ class UIControl(metaclass=ABCMeta):
         width: int,
         max_available_height: int,
         wrap_lines: bool,
-        get_line_prefix: Optional[GetLinePrefixCallable],
-    ) -> Optional[int]:
+        get_line_prefix: GetLinePrefixCallable | None,
+    ) -> int | None:
         return None
 
     def is_focusable(self) -> bool:
@@ -106,14 +87,14 @@ class UIControl(metaclass=ABCMeta):
         return False
 
     @abstractmethod
-    def create_content(self, width: int, height: int) -> "UIContent":
+    def create_content(self, width: int, height: int) -> UIContent:
         """
         Generate the content for this user control.
 
         Returns a :class:`.UIContent` instance.
         """
 
-    def mouse_handler(self, mouse_event: MouseEvent) -> "NotImplementedOrNone":
+    def mouse_handler(self, mouse_event: MouseEvent) -> NotImplementedOrNone:
         """
         Handle mouse events.
 
@@ -137,7 +118,7 @@ class UIControl(metaclass=ABCMeta):
         Request to move the cursor up.
         """
 
-    def get_key_bindings(self) -> Optional["KeyBindingsBase"]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         """
         The key bindings that are specific for this user control.
 
@@ -145,7 +126,7 @@ class UIControl(metaclass=ABCMeta):
         specified, or `None` otherwise.
         """
 
-    def get_invalidate_events(self) -> Iterable["Event[object]"]:
+    def get_invalidate_events(self) -> Iterable[Event[object]]:
         """
         Return a list of `Event` objects. This can be a generator.
         (The application collects all these events, in order to bind redraw
@@ -171,11 +152,10 @@ class UIContent:
         self,
         get_line: Callable[[int], StyleAndTextTuples] = (lambda i: []),
         line_count: int = 0,
-        cursor_position: Optional[Point] = None,
-        menu_position: Optional[Point] = None,
+        cursor_position: Point | None = None,
+        menu_position: Point | None = None,
         show_cursor: bool = True,
     ):
-
         self.get_line = get_line
         self.line_count = line_count
         self.cursor_position = cursor_position or Point(x=0, y=0)
@@ -183,7 +163,7 @@ class UIContent:
         self.show_cursor = show_cursor
 
         # Cache for line heights. Maps cache key -> height
-        self._line_heights_cache: Dict[Hashable, int] = {}
+        self._line_heights_cache: dict[Hashable, int] = {}
 
     def __getitem__(self, lineno: int) -> StyleAndTextTuples:
         "Make it iterable (iterate line by line)."
@@ -196,8 +176,8 @@ class UIContent:
         self,
         lineno: int,
         width: int,
-        get_line_prefix: Optional[GetLinePrefixCallable],
-        slice_stop: Optional[int] = None,
+        get_line_prefix: GetLinePrefixCallable | None,
+        slice_stop: int | None = None,
     ) -> int:
         """
         Return the height that a given line would need if it is rendered in a
@@ -219,7 +199,7 @@ class UIContent:
             return self._line_heights_cache[key]
         except KeyError:
             if width == 0:
-                height = 10 ** 8
+                height = 10**8
             else:
                 # Calculate line width first.
                 line = fragment_list_to_text(self.get_line(lineno))[:slice_stop]
@@ -246,7 +226,7 @@ class UIContent:
                         prefix_width = get_cwidth(fragment_list_to_text(fragments2))
 
                         if prefix_width >= width:  # Prefix doesn't fit.
-                            height = 10 ** 8
+                            height = 10**8
                             break
 
                         text_width += prefix_width
@@ -255,7 +235,7 @@ class UIContent:
                     try:
                         quotient, remainder = divmod(text_width, width)
                     except ZeroDivisionError:
-                        height = 10 ** 8
+                        height = 10**8
                     else:
                         if remainder:
                             quotient += 1  # Like math.ceil.
@@ -314,12 +294,11 @@ class FormattedTextControl(UIControl):
         text: AnyFormattedText = "",
         style: str = "",
         focusable: FilterOrBool = False,
-        key_bindings: Optional["KeyBindingsBase"] = None,
+        key_bindings: KeyBindingsBase | None = None,
         show_cursor: bool = True,
         modal: bool = False,
-        get_cursor_position: Optional[Callable[[], Optional[Point]]] = None,
+        get_cursor_position: Callable[[], Point | None] | None = None,
     ) -> None:
-
         self.text = text  # No type check on 'text'. This is done dynamically.
         self.style = style
         self.focusable = to_filter(focusable)
@@ -338,7 +317,7 @@ class FormattedTextControl(UIControl):
         # Only cache one fragment list. We don't need the previous item.
 
         # Render info for the mouse support.
-        self._fragments: Optional[StyleAndTextTuples] = None
+        self._fragments: StyleAndTextTuples | None = None
 
     def reset(self) -> None:
         self._fragments = None
@@ -347,7 +326,7 @@ class FormattedTextControl(UIControl):
         return self.focusable()
 
     def __repr__(self) -> str:
-        return "%s(%r)" % (self.__class__.__name__, self.text)
+        return f"{self.__class__.__name__}({self.text!r})"
 
     def _get_formatted_text_cached(self) -> StyleAndTextTuples:
         """
@@ -373,8 +352,8 @@ class FormattedTextControl(UIControl):
         width: int,
         max_available_height: int,
         wrap_lines: bool,
-        get_line_prefix: Optional[GetLinePrefixCallable],
-    ) -> Optional[int]:
+        get_line_prefix: GetLinePrefixCallable | None,
+    ) -> int | None:
         """
         Return the preferred height for this control.
         """
@@ -389,7 +368,7 @@ class FormattedTextControl(UIControl):
         else:
             return content.line_count
 
-    def create_content(self, width: int, height: Optional[int]) -> UIContent:
+    def create_content(self, width: int, height: int | None) -> UIContent:
         # Get fragments
         fragments_with_mouse_handlers = self._get_formatted_text_cached()
         fragment_lines_with_mouse_handlers = list(
@@ -397,7 +376,7 @@ class FormattedTextControl(UIControl):
         )
 
         # Strip mouse handlers from fragments.
-        fragment_lines: List[StyleAndTextTuples] = [
+        fragment_lines: list[StyleAndTextTuples] = [
             [(item[0], item[1]) for item in line]
             for line in fragment_lines_with_mouse_handlers
         ]
@@ -410,7 +389,7 @@ class FormattedTextControl(UIControl):
         # cursor position here.
         def get_cursor_position(
             fragment: str = "[SetCursorPosition]",
-        ) -> Optional[Point]:
+        ) -> Point | None:
             for y, line in enumerate(fragment_lines):
                 x = 0
                 for style_str, text, *_ in line:
@@ -420,7 +399,7 @@ class FormattedTextControl(UIControl):
             return None
 
         # If there is a `[SetMenuPosition]`, set the menu over here.
-        def get_menu_position() -> Optional[Point]:
+        def get_menu_position() -> Point | None:
             return get_cursor_position("[SetMenuPosition]")
 
         cursor_position = (self.get_cursor_position or get_cursor_position)()
@@ -439,7 +418,7 @@ class FormattedTextControl(UIControl):
 
         return self._content_cache.get(key, get_content)
 
-    def mouse_handler(self, mouse_event: MouseEvent) -> "NotImplementedOrNone":
+    def mouse_handler(self, mouse_event: MouseEvent) -> NotImplementedOrNone:
         """
         Handle mouse events.
 
@@ -470,7 +449,7 @@ class FormattedTextControl(UIControl):
                             # Handler found. Call it.
                             # (Handler can return NotImplemented, so return
                             # that result.)
-                            handler = item[2]  # type: ignore
+                            handler = item[2]
                             return handler(mouse_event)
                         else:
                             break
@@ -481,7 +460,7 @@ class FormattedTextControl(UIControl):
     def is_modal(self) -> bool:
         return self.modal
 
-    def get_key_bindings(self) -> Optional["KeyBindingsBase"]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         return self.key_bindings
 
 
@@ -498,22 +477,16 @@ class DummyControl(UIControl):
         def get_line(i: int) -> StyleAndTextTuples:
             return []
 
-        return UIContent(
-            get_line=get_line, line_count=100 ** 100
-        )  # Something very big.
+        return UIContent(get_line=get_line, line_count=100**100)  # Something very big.
 
     def is_focusable(self) -> bool:
         return False
 
 
-_ProcessedLine = NamedTuple(
-    "_ProcessedLine",
-    [
-        ("fragments", StyleAndTextTuples),
-        ("source_to_display", Callable[[int], int]),
-        ("display_to_source", Callable[[int], int]),
-    ],
-)
+class _ProcessedLine(NamedTuple):
+    fragments: StyleAndTextTuples
+    source_to_display: Callable[[int], int]
+    display_to_source: Callable[[int], int]
 
 
 class BufferControl(UIControl):
@@ -538,20 +511,19 @@ class BufferControl(UIControl):
 
     def __init__(
         self,
-        buffer: Optional[Buffer] = None,
-        input_processors: Optional[List[Processor]] = None,
+        buffer: Buffer | None = None,
+        input_processors: list[Processor] | None = None,
         include_default_input_processors: bool = True,
-        lexer: Optional[Lexer] = None,
+        lexer: Lexer | None = None,
         preview_search: FilterOrBool = False,
         focusable: FilterOrBool = True,
-        search_buffer_control: Union[
-            None, "SearchBufferControl", Callable[[], "SearchBufferControl"]
-        ] = None,
-        menu_position: Optional[Callable[[], Optional[int]]] = None,
+        search_buffer_control: (
+            None | SearchBufferControl | Callable[[], SearchBufferControl]
+        ) = None,
+        menu_position: Callable[[], int | None] | None = None,
         focus_on_click: FilterOrBool = False,
-        key_bindings: Optional["KeyBindingsBase"] = None,
+        key_bindings: KeyBindingsBase | None = None,
     ):
-
         self.input_processors = input_processors
         self.include_default_input_processors = include_default_input_processors
 
@@ -580,15 +552,15 @@ class BufferControl(UIControl):
             Hashable, Callable[[int], StyleAndTextTuples]
         ] = SimpleCache(maxsize=8)
 
-        self._last_click_timestamp: Optional[float] = None
-        self._last_get_processed_line: Optional[Callable[[int], _ProcessedLine]] = None
+        self._last_click_timestamp: float | None = None
+        self._last_get_processed_line: Callable[[int], _ProcessedLine] | None = None
 
     def __repr__(self) -> str:
-        return "<%s buffer=%r at %r>" % (self.__class__.__name__, self.buffer, id(self))
+        return f"<{self.__class__.__name__} buffer={self.buffer!r} at {id(self)!r}>"
 
     @property
-    def search_buffer_control(self) -> Optional["SearchBufferControl"]:
-        result: Optional[SearchBufferControl]
+    def search_buffer_control(self) -> SearchBufferControl | None:
+        result: SearchBufferControl | None
 
         if callable(self._search_buffer_control):
             result = self._search_buffer_control()
@@ -599,7 +571,7 @@ class BufferControl(UIControl):
         return result
 
     @property
-    def search_buffer(self) -> Optional[Buffer]:
+    def search_buffer(self) -> Buffer | None:
         control = self.search_buffer_control
         if control is not None:
             return control.buffer
@@ -622,7 +594,7 @@ class BufferControl(UIControl):
     def is_focusable(self) -> bool:
         return self.focusable()
 
-    def preferred_width(self, max_available_width: int) -> Optional[int]:
+    def preferred_width(self, max_available_width: int) -> int | None:
         """
         This should return the preferred width.
 
@@ -640,9 +612,8 @@ class BufferControl(UIControl):
         width: int,
         max_available_height: int,
         wrap_lines: bool,
-        get_line_prefix: Optional[GetLinePrefixCallable],
-    ) -> Optional[int]:
-
+        get_line_prefix: GetLinePrefixCallable | None,
+    ) -> int | None:
         # Calculate the content height, if it was drawn on a screen with the
         # given width.
         height = 0
@@ -672,6 +643,7 @@ class BufferControl(UIControl):
         """
         Create a function that returns the fragments for a given line.
         """
+
         # Cache using `document.text`.
         def get_formatted_text_for_line() -> Callable[[int], StyleAndTextTuples]:
             return self.lexer.lex_document(document)
@@ -696,6 +668,7 @@ class BufferControl(UIControl):
 
         def transform(lineno: int, fragments: StyleAndTextTuples) -> _ProcessedLine:
             "Transform the fragments for a given line number."
+
             # Get cursor position at this line.
             def source_to_display(i: int) -> int:
                 """X position from the buffer to the x position in the
@@ -717,7 +690,7 @@ class BufferControl(UIControl):
 
         def create_func() -> Callable[[int], _ProcessedLine]:
             get_line = self._get_formatted_text_for_line_func(document)
-            cache: Dict[int, _ProcessedLine] = {}
+            cache: dict[int, _ProcessedLine] = {}
 
             def get_processed_line(i: int) -> _ProcessedLine:
                 try:
@@ -836,7 +809,7 @@ class BufferControl(UIControl):
 
         return content
 
-    def mouse_handler(self, mouse_event: MouseEvent) -> "NotImplementedOrNone":
+    def mouse_handler(self, mouse_event: MouseEvent) -> NotImplementedOrNone:
         """
         Mouse handler for this control.
         """
@@ -858,8 +831,15 @@ class BufferControl(UIControl):
                     buffer.exit_selection()
                     buffer.cursor_position = index
 
-                elif mouse_event.event_type == MouseEventType.MOUSE_DOWN_MOVE:
-                    if buffer.selection_state is None:
+                elif (
+                    mouse_event.event_type == MouseEventType.MOUSE_MOVE
+                    and mouse_event.button != MouseButton.NONE
+                ):
+                    # Click and drag to highlight a selection
+                    if (
+                        buffer.selection_state is None
+                        and abs(buffer.cursor_position - index) > 0
+                    ):
                         buffer.start_selection(selection_type=SelectionType.CHARACTERS)
                     buffer.cursor_position = index
 
@@ -916,13 +896,13 @@ class BufferControl(UIControl):
         b = self.buffer
         b.cursor_position += b.document.get_cursor_up_position()
 
-    def get_key_bindings(self) -> Optional["KeyBindingsBase"]:
+    def get_key_bindings(self) -> KeyBindingsBase | None:
         """
         When additional key bindings are given. Return these.
         """
         return self.key_bindings
 
-    def get_invalidate_events(self) -> Iterable["Event[object]"]:
+    def get_invalidate_events(self) -> Iterable[Event[object]]:
         """
         Return the Window invalidate events.
         """
@@ -944,14 +924,13 @@ class SearchBufferControl(BufferControl):
 
     def __init__(
         self,
-        buffer: Optional[Buffer] = None,
-        input_processors: Optional[List[Processor]] = None,
-        lexer: Optional[Lexer] = None,
+        buffer: Buffer | None = None,
+        input_processors: list[Processor] | None = None,
+        lexer: Lexer | None = None,
         focus_on_click: FilterOrBool = False,
-        key_bindings: Optional["KeyBindingsBase"] = None,
+        key_bindings: KeyBindingsBase | None = None,
         ignore_case: FilterOrBool = False,
     ):
-
         super().__init__(
             buffer=buffer,
             input_processors=input_processors,
